/* memory.v - implementation of memory external to the 9086 CPU for testing purposes

   This file is part of the 9086 project.

   Copyright (c) 2023 Efthymios Kritikos

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */

module doublemem(input [19:0] address,inout wire [15:0] data ,input rd,input wr,input BHE,input cs);
reg [15:0] memory [0:32768];
initial begin
	string boot_code;
	if(!$value$plusargs("BOOT_CODE=%s",boot_code))begin
		$display("No boot code specified. Please add +BOOT_CODE=<path> to your vvp args");
		$finish;
	end
	$readmemh(boot_code, memory,0,16383);
end

assign data[7:0] = !address[0:0] & !rd & !cs ? memory[address[15:1]][15:8]  : 16'hz;

assign data[15:8] =     !BHE     & !rd & !cs ? memory[address[15:1]][7:0] : 16'hz;

always @(negedge wr) begin
	if(BHE==0)
		memory[address[15:1]][7:0]=data[15:8];
	if(address[0]==0)
		memory[address[15:1]][15:8]=data[7:0];
end

endmodule
